'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _update = require('react/lib/update');

var _update2 = _interopRequireDefault(_update);

var _objectAssign = require('object-assign');

var _objectAssign2 = _interopRequireDefault(_objectAssign);

var _eventEmitter = require('event-emitter');

var _eventEmitter2 = _interopRequireDefault(_eventEmitter);

var _data = require('./data');

var _data2 = _interopRequireDefault(_data);

var _utils = require('./utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

exports.default = function (Component) {
  var _ref = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

  var _ref$options = _ref.options;
  var options = _ref$options === undefined ? {} : _ref$options;
  var _ref$queryParams = _ref.queryParams;
  var queryParams = _ref$queryParams === undefined ? {} : _ref$queryParams;
  var _ref$queries = _ref.queries;
  var queries = _ref$queries === undefined ? {} : _ref$queries;
  var _ref$mutations = _ref.mutations;
  var mutations = _ref$mutations === undefined ? {} : _ref$mutations;

  var displayName = (0, _utils.getContainerName)(Component);
  var queryRequests = (0, _utils.getRequestsFromQueries)(queries);
  var mutationRequests = (0, _utils.getRequestsFromQueries)(mutations);
  var initialRequestNames = queryRequests.map(function (_ref2) {
    var name = _ref2.name;
    return name;
  });
  var initialState = (0, _objectAssign2.default)({ $pendingRequests: initialRequestNames }, (0, _utils.getInitialStateFromRequests)(queryRequests));

  return _react2.default.createClass({
    displayName: displayName,

    getInitialState: function getInitialState() {
      return initialState;
    },
    componentWillMount: function componentWillMount() {
      this.queryParams = (0, _update2.default)(queryParams, { $merge: this.props });
      this.ee = (0, _eventEmitter2.default)();
      this.fetcher = (0, _data2.default)(options);
      this.unregister = this.fetcher.onReceive(this.onReceive);
    },
    componentDidMount: function componentDidMount() {
      this.fetch(queryRequests);
    },
    componentWillReceiveProps: function componentWillReceiveProps(nextProps) {
      this.setQueryParams(nextProps);
    },
    shouldComponentUpdate: function shouldComponentUpdate(nextProps, nextState) {
      var _this = this;

      if (nextState.$pendingRequests.length) {
        return false;
      }
      return Object.keys(nextProps).some(function (key) {
        return nextProps[key] !== _this.props[key];
      }) || Object.keys(nextState).some(function (key) {
        return nextState[key] !== _this.state[key];
      });
    },
    componentWillUnmount: function componentWillUnmount() {
      this.ee = undefined;
      this.unregister();
    },
    onReceive: function onReceive(_ref3) {
      var request = _ref3.request;
      var requestData = _ref3.data;

      if (this.isMounted && !this.isMounted()) {
        return;
      }
      var i = this.state.$pendingRequests.indexOf(request.name);
      var newState = _defineProperty({}, request.name, { $set: requestData });
      if (i !== -1) {
        newState.$pendingRequests = { $splice: [[i, 1]] };
      }
      this.setState((0, _update2.default)(this.state, newState));
    },
    fetch: function fetch(requests) {
      var _this2 = this;

      var opt = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

      requests.forEach(function (request) {
        var params = (0, _utils.setParamValues)(request.params, _this2.queryParams);
        _this2.fetcher.fetch({ request: request, params: params, options: opt });
      });
    },
    mutate: function mutate(request, requestData) {
      var qp = (0, _objectAssign2.default)({}, this.queryParams, requestData);
      var params = (0, _utils.setParamValues)(request.params, qp);
      this.fetcher.run({ request: request, params: params, data: requestData, options: options });
    },
    setQueryParams: function setQueryParams(newParams) {
      var _this3 = this;

      var paramNames = (0, _utils.getParamNames)(newParams);
      var requestsToRedo = queryRequests.filter(function (request) {
        return paramNames.some(function (name) {
          return request.paramDependencies.indexOf(name) !== -1;
        });
      });
      this.queryParams = (0, _update2.default)(this.queryParams, { $merge: newParams });
      if (requestsToRedo.length) {
        this.setState((0, _update2.default)(this.state, {
          $pendingRequests: {
            $push: requestsToRedo.filter(function (_ref4) {
              var name = _ref4.name;
              return _this3.state.$pendingRequests.indexOf(name) === -1;
            }).map(function (_ref5) {
              var name = _ref5.name;
              return name;
            })
          }
        }));
      }
      this.fetch(requestsToRedo, { force: true });
    },
    render: function render() {
      var _this4 = this;

      this.ee.emit('render');

      if (this.state.$pendingRequests.length) {
        return null;
      }

      var mutators = mutationRequests.reduce(function (mutatorProps, mutation) {
        return (0, _objectAssign2.default)(_defineProperty({}, mutation.name, _this4.mutate.bind(_this4, mutation)), mutatorProps);
      }, {});

      var setQueryParams = this.setQueryParams;

      var props = (0, _objectAssign2.default)({ setQueryParams: setQueryParams }, this.props, this.state, mutators);

      return _react2.default.createElement(Component, props);
    }
  });
};